const { objectsAreEqual } = require("./utl");

/*
## No influence on cache
  -i, --info                  
  -V, --version               
  -h, --help         
  -T, --output-type <type>    unless counting implicit shizzle like dependendents, metrics calculation
  -f, --output-to <file>      
  -p, --progress [type]       
  -P, --prefix <prefix>       
  -C, --cache [cache-location]
 --init [oneshot]             


## Influence on cache
### 100% invalidate when different
  -c, --config [file]         
  --ignore-known [file]       
  --ts-config [file]          
  --webpack-config [file]     
  --ts-pre-compilation-deps   
  -v, --validate [file]       
  --preserve-symlinks         

### Invalidates when in cache, not in command 
(or more precise: when filters in cache yield a subset of command)
  -I, --include-only <regex>  
  -F, --focus <regex>         
  -x, --exclude <regex>       
  -X, --do-not-follow <regex> 
  -S, --collapse <regex>      
  -d, --max-depth <n>          (more precise: when cache.max-depth < command.max-depth OR cache.max-depth === 0)
  -M, --module-systems <items> (more precise: )

### Invalidates when not in cache, but in command
  -m, --metrics               
*/

function includeOnlyFiltersAreCompatible(pExistingFilter, pNewFilter) {
  return (
    !pExistingFilter || objectsAreEqual({ path: pExistingFilter }, pNewFilter)
  );
}
function filtersAreCompatible(pExistingFilter, pNewFilter) {
  return !pExistingFilter || objectsAreEqual(pExistingFilter, pNewFilter);
}

function limitsAreCompatible(pExistingLimit, pNewLimit) {
  return !pExistingLimit || pExistingLimit >= (pNewLimit || pExistingLimit + 1);
}

/**
 *
 * @param {import("../../types/strict-options").IStrictCruiseOptions} pOldOptions
 * @param {import("../../types/strict-options").IStrictCruiseOptions} pNewOptions
 * @returns {boolean}
 */
// eslint-disable-next-line complexity
function optionsAreCompatible(pOldOptions, pNewOptions) {
  return (
    pOldOptions.args === pNewOptions.args &&
    pOldOptions.rulesFile === pNewOptions.rulesFile &&
    pOldOptions.tsPreCompilationDeps === pNewOptions.tsPreCompilationDeps &&
    pOldOptions.preserveSymlinks === pNewOptions.preserveSymlinks &&
    // pOldOptions.metrics === pNewOptions.metrics &&
    // includeOnly suffers from a backwards compatibility disease
    includeOnlyFiltersAreCompatible(
      pOldOptions.includeOnly,
      pNewOptions.includeOnly
    ) &&
    filtersAreCompatible(pOldOptions.exclude, pNewOptions.exclude) &&
    filtersAreCompatible(pOldOptions.focus, pNewOptions.focus) &&
    filtersAreCompatible(pOldOptions.doNotFollow, pNewOptions.doNotFollow) &&
    filtersAreCompatible(pOldOptions.collapse, pNewOptions.collapse) &&
    limitsAreCompatible(pOldOptions.maxDepth, pNewOptions.maxDepth) &&
    filtersAreCompatible(pOldOptions.moduleSystems, pNewOptions.moduleSystems)
  );
}

module.exports = {
  optionsAreCompatible,
  filtersAreCompatible,
  limitsAreCompatible,
  includeOnlyFiltersAreCompatible,
};
